package rescuecore.map09.com;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import rescuecore.Agent;
import rescuecore.RescueConstants;

public class CommInterface {
	public final static byte NO_TOKEN = 0x00;
	// IDTokens
    public final static byte EXPLORED_ID = 0x01;
    // IDListTokens
    public final static byte EXPLORED_ID_LIST = 0x20;
	
	protected Agent myAgent;
	protected List<CommObject> sendQueue;

	public CommInterface(Agent myAgent) {
		this.myAgent = myAgent;
		sendQueue = new ArrayList<CommObject>();
	}

	public void add(CommObject obj) {
		sendQueue.add(obj);
	}
	

	public void tellAll() {
		byte [] msg = new byte[RescueConstants.TELL_LENGTH];

		int start = 0;
		for(CommObject obj : sendQueue) {
			int ret = encode(msg, start, obj);
			start = ret;
			if(ret == -1)
				break;
			//TODO: Distribute to MAX_..._MESSAGES (from RescueConstants), 
			// if sendQueue doesn't fit in one msg
		}
		//TODO: Seperate CommObjects to the channels given and send to correct channel
		if(start != -1)
			tell(0, msg);
		sendQueue.clear();
	}
	
	public boolean willListenHear(int from) {
		//TODO: Listen to center, if myAgent is Platoon and vice versa
		return true;
	}

	public Collection<CommObject> hear(int from, byte[] msg) {
		ArrayList<CommObject> ret = new ArrayList<CommObject>();
		System.out.println("Hearing...");
		for (int i = 0; i < msg.length; i++) {
			System.out.print(msg[i]);
		}
		System.out.println("");

		int start = 0;
		if (start >= msg.length)
			return ret;
		byte type = msg[start];
		start++;
		CommObject obj = newToken(type);
		if (obj == null)
			return ret;
		//TODO: decode multiple messages
		start = obj.decode(msg, start);
		if (start == -1)
			return ret;

		if (obj != null)
			ret.add(obj);
		return ret;
	}

	protected final void tell(int channel, byte[] message) {
		myAgent.tell(channel, message);
	}

	public final int encode(byte [] msg, int start, CommObject obj) {
		if(start >= msg.length)
			return -1;
		msg[start] = obj.getType();
		start ++;
		int ret = obj.encode(msg, start);
		return ret;
	}
	
	protected CommObject newToken(byte type) {
		switch(type) {
			case EXPLORED_ID:
				return new IDToken(type);
		}
		return null;
	}
}
